# coding: utf-8
"""
class StationmetaCore (Base)
============================
"""
from sqlalchemy import Column, DateTime, Float, ForeignKey, Integer, String, \
                       Text, CheckConstraint, Sequence
from geoalchemy2.types import Geometry
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import JSONB, ARRAY
from shapely import wkt
from toardb.auth_user.models import AuthUser
from toardb.base import Base

STATIONMETA_CORE_ID_SEQ = Sequence('stationmeta_core_id_seq')  # define sequence explicitly
# coordinates are giving problems in nested views (from timeseries -- now try this approach)
# ==> unfortunately, there will be no coordinates shown in timeseries nested views
# still to be investigated!
#   ValueError: [ValueError('dictionary update sequence element #0 has length 1; 2 is required',), TypeError('vars() argument must have __dict__ attribute',)]
class StationmetaCore_WithoutCoords(Base):
    """ Table "public.stationmeta_core"

    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    |            Column            |           Type           | Collation | Nullable |                   Default                    |
    +==============================+==========================+===========+==========+==============================================+
    | id                           | integer                  |           | not null | nextval('stationmeta_core_id_seq'::regclass) |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | codes                        | character varying(16)[]  |           |          |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | name                         | character varying(128)   |           | not null |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | coordinates                  | geometry(PointZ,4326)    |           | not null |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | country                      | character varying(128)   |           | not null |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | state                        | character varying(128)   |           | not null |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | type_of_environment          | integer                  |           | not null |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | type_of_area                 | integer                  |           | not null |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | timezone                     | character varying(64)    |           | not null |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    | additional_metadata          | jsonb                    |           | not null |                                              |
    +------------------------------+--------------------------+-----------+----------+----------------------------------------------+
    Indexes:
        "stationmeta_core_pkey" PRIMARY KEY, btree (id)
        "stationmeta_core_coordinates_id" gist (coordinates gist_geometry_ops_nd)
        "stationmeta_core_country_fa755dde" btree (country)
        "stationmeta_core_country_fa755dde_like" btree (country varchar_pattern_ops)
        "stationmeta_core_state_85025a96" btree (state)
        "stationmeta_core_state_85025a96_like" btree (state varchar_pattern_ops)
    Check constraints:
        "stationmeta_core_type_of_area_check" CHECK (type_of_area >= 0)
        "stationmeta_core_type_of_environment_check" CHECK (type_of_environment >= 0)
    Foreign-key constraints:
        "stationmeta_core_type_of_area_fk_ta_vocabulary_enum_val" FOREIGN KEY (type_of_area) REFERENCES ta_vocabulary(enum_val)
        "stationmeta_core_type_of_environment_fk_st_vocabulary_enum_val" FOREIGN KEY (type_of_environment) REFERENCES st_vocabulary(enum_val)
    Referenced by:
        TABLE "station_roles" CONSTRAINT "station_roles_station_id_f31f80fc_fk_stationmeta_core_id" FOREIGN KEY (station_id) REFERENCES stationmeta_core(id) DEFERRABLE INITIALLY DEFERRED
        TABLE "stationmeta_annotations" CONSTRAINT "stationmeta_annotati_station_id_9d3fe3d0_fk_stationme" FOREIGN KEY (station_id) REFERENCES stationmeta_core(id) DEFERRABLE INITIALLY DEFERRED
        TABLE "stationmeta_aux_doc" CONSTRAINT "stationmeta_aux_doc_station_id_17bdb5f2_fk_stationmeta_core_id" FOREIGN KEY (station_id) REFERENCES stationmeta_core(id) DEFERRABLE INITIALLY DEFERRED
        TABLE "stationmeta_aux_image" CONSTRAINT "stationmeta_aux_imag_station_id_fbfbdb29_fk_stationme" FOREIGN KEY (station_id) REFERENCES stationmeta_core(id) DEFERRABLE INITIALLY DEFERRED
        TABLE "stationmeta_aux_url" CONSTRAINT "stationmeta_aux_url_station_id_727571bd_fk_stationmeta_core_id" FOREIGN KEY (station_id) REFERENCES stationmeta_core(id) DEFERRABLE INITIALLY DEFERRED
        TABLE "stationmeta_global" CONSTRAINT "stationmeta_global_station_id_29ff53dd_fk_stationmeta_core_id" FOREIGN KEY (station_id) REFERENCES stationmeta_core(id) DEFERRABLE INITIALLY DEFERRED
        TABLE "timeseries" CONSTRAINT "timeseries_station_id_0f4fed9c_fk_stationmeta_core_id" FOREIGN KEY (station_id) REFERENCES stationmeta_core(id) DEFERRABLE INITIALLY DEFERRED
    """

    __tablename__ = 'stationmeta_core'
    __table_args__ = (
                        CheckConstraint('type_of_area >= 0'),
                        CheckConstraint('type_of_environment >= 0')
                     )

    id = Column(Integer, STATIONMETA_CORE_ID_SEQ, primary_key=True, server_default=STATIONMETA_CORE_ID_SEQ.next_value())
    codes = Column(ARRAY(String(16)))
    name = Column(String(128), nullable=False)
    country = Column(String(128), nullable=False, index=True)
    state = Column(String(128), nullable=False, index=True)
    type_of_environment = Column(ForeignKey('st_vocabulary.enum_val'), nullable=False)
    type_of_area = Column(ForeignKey('ta_vocabulary.enum_val'), nullable=False)
    timezone = Column(String(64), nullable=False)
    additional_metadata = Column(JSONB(astext_type=Text()), nullable=True)

    aux_images = relationship('StationmetaAuxImage', back_populates='station')
    aux_docs = relationship('StationmetaAuxDoc', back_populates='station')
    aux_urls = relationship('StationmetaAuxUrl', back_populates='station')
    globalmeta = relationship('StationmetaGlobal', uselist=False, back_populates='station')
    globalservice = relationship('StationmetaGlobalService', uselist=False, back_populates='station')

    # for nested view
    timeseries = relationship("Timeseries", back_populates="station")

#   cv_vocabulary = relationship('CvVocabulary')
#   ta_vocabulary = relationship('TaVocabulary')
#   st_vocabulary = relationship('StVocabulary')

class StationmetaCore(StationmetaCore_WithoutCoords):
    coordinates = Column(Geometry('POINTZ', 4326))
